import { clone, diff, fail, succeed, Tuple } from './fn.js'
import { anyChar } from './char.js'
import { ok } from 'kojima'
import { curry } from 'izuna'

/** @import { ParseError, ParserState } from './state.js' */
/** @import { Result } from '../vendor/kojima/src/index.js' */

/**
 * @typedef {(value: any) => Result<ParserState, ParseError>} Parser
 */

export const take = n => state => {
  let result = anyChar(state)
  for (let i = n; i > 0; i--) {
    if (result.isErr()) {
      return result.chain(e => fail(`"take(${n})" failed`, state, e))
    }

    result = result.chain(anyChar)
  }

  return result
}

export const skip = parser => state => {
  const tokens = state[0]
  const result = parser(state)
  if (result.isOk()) {
    return result.map(other => [tokens, other[1]])
  } else {
    return result
  }
}

/**
 * @param {...Parser} parsers
 */
export const seq = (...parsers) =>
  /** @param {ParserState} state */
  state => {
    let acc = ok(state)
    for (const parser of parsers) {
      if (acc.isOk()) {
        acc = acc.chain(parser)
      } else {
        break
      }
    }

    return acc
  }


export const many = curry((parser, state) => {
  let result = ok(state)

  while (true) {
    const res = parser(clone(state))
    if (res.isOk()) {
      result = res
    } else {
      break
    }
  }

  return result
})

export const many1 = parser => seq(parser, many(parser))

