import { Iter } from './iter.js'
import { seq } from './seq.js'
import { until } from './cond.js'
import { eof } from './combinator.js'
import { curry, pipe } from 'izuna'

/**
 * @typedef {Readonly<[any[], Iterator<any>]>} ParserState
 */

export class ParseError extends Error {
  /**
   * @param {string} message
   * @param {ParserState} state
   * @param {Error} [cause]
   */
  constructor(message, state, cause) {
    super(message, { cause })
    this.state = state
  }
}

/**
 * @param {any} value
 * @returns {ParserState}
 */
export const State = value => Object.freeze([[], Iter.from(value)])

export const parse = curry((parser, input) => parser(State(input)))
export const parseAll = curry((parser, input) => pipe(
  State,
  seq(
    parser,
    until(eof)
  )
)(input))

